import { Injectable } from '@angular/core';
import { Http, Headers, Response, RequestOptions, ResponseContentType } from '@angular/http';
import { TranslateService } from '@ngx-translate/core';
import { Observable } from 'rxjs/Observable';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/toPromise';

/**
 * Report descriptor.
 */
export class Report {
	constructor(
		public name: string,
		public id: string,
		public isParametrized: boolean
	) { }
}

/**
 * Printer descriptor.
 */
export class Printer {
	constructor(
		public title: string,
		public id: string,
		public agentId: string,
		public canDuplex: boolean,
		public twoSided: TwoSided,
		public maximumCopies: number,
		public supportsColor: boolean,
		public paperSizes: Array<string>,
		public paperSources: Array<string>
	) { }
}

/**
 * 'Two sided' option descriptor.
 */
export enum TwoSided {
	OneSided,
	LongEdge,
	ShortEdge
}

/**
 * ActiveReports Server RESTful API calls.
 */
@Injectable()
export class ArsRestService {

	private _headers: Headers;
	private _reportId: string;
	private _documentId: string;

	constructor(
		private _http: Http,
		private _translate: TranslateService) {

		this._headers = new Headers({
			'Content-Type': 'application/json; charset=utf-8'
		});
	}

	/**
	 * Login to server and preserve security token in session.
	 */
	login(server: string, username: string, password: string): Promise<void> {
		this.logout();

		if (!server) {
			return Promise.reject(this._translate.instant('ERRORS.NO_SERVER_URL'));
		}
		if (server.lastIndexOf('/') !== server.length - 1)
			server = server + '/';

		// send login request
		this._headers.set('AuthToken', '');
		var data = { User: username && username.length > 0 ? username : 'Anonymous', Password: password };
		return this._http.post(server + 'api/accounts/login', data, new RequestOptions({ headers: this._headers }))
			.map((value: Response) => {
				var tokenData = value.json();
				if (!tokenData)
					return;
				var token = tokenData.Token.toString();
				// use localStorage to preserve values in session
				localStorage.setItem('ars.token', token);
				localStorage.setItem('ars.server', server);
			})
			.toPromise()
			.catch((reason: any) => Promise.reject(this.parseError(reason)));
	}

	/**
	 * Logout.
	 */
	logout(): void {
		this._reportId = null;
		this._documentId = null;
		var token = localStorage.getItem('ars.token');
		if (!token)
			return;
		this.sendPostRequest('api/accounts/logout', { Token: token });

		localStorage.removeItem('ars.token');
		localStorage.removeItem('ars.server');
	}

	/**
	 * Checks login state.
	 */
	hasCredentials(): boolean {
		return !!localStorage.getItem('ars.token');
	}

	/**
	 * Checks credentials.
	 */
	isCredentialsValid(): Promise<void> {
		var token = localStorage.getItem('ars.token');
		if (!token)
			return Promise.reject(this._translate.instant('REST_API.CREDENTIALS.NOT_LOGGED_MESSAGE'));
		var classThis = this;
		return new Promise<void>((resolve, reject) => {
			classThis.sendGetRequest('api/accounts/check').then(
				(data: Response) => {
					if (data.json().Valid)
						resolve();
					else
						reject(this._translate.instant('REST_API.CREDENTIALS.NOT_VALID_CREDENTIALS_MESSAGE'));
				},
				(err: any) => {
					reject(err);
				});
		});
	}

	/**
	 * Gets all reports.
	 */
	get reports(): Promise<Report[]> {
		return this.sendGetRequest('api/reports').then(
			(data: Response) => {
				var reportsList = new Array<Report>();
				for (var item of <Array<any>>data.json()) {
					reportsList.push(new Report(item['Name'], item['_id'], item['IsParametrized']));
				}
				return reportsList;
			});
	}

	/**
	 * Renders report and exports to return URL.
	 * @param id - reports ID
	 * @param extensionName - possible extension name: 'PDF', 'Word', 'Image', 'Mht', 'Excel'
	 * @param printDialog - open print dialog, works for PDF only
	 * @param parameters - report parameter values array
	 */
	render(id: string, extensionName: string, printDialog: boolean, parameters: Array<any>): Promise<ArrayBuffer> {
		this._documentId = null; // reset export identifier
		this._reportId = null;

		var classThis = this;
		var renderSettings = classThis.getExportSettings(extensionName, false, printDialog);
		renderSettings.NeedExportSupport = true; // we need to render first because we need to export...

		return new Promise<ArrayBuffer>((resolve, reject) => {
			classThis.handleRepeatRequest(id, 'reports', 'render', 'reports/jobs/render', '',
				{ ExtensionName: 'Pdf', ExtensionSettings: renderSettings, UseCache: false, ReportParameters: parameters || new Array<any>() }, ResponseContentType.ArrayBuffer).then(
				(data: Response) => {
					classThis._documentId = data.headers.get('X-ARS-DocumentId');
					if (!classThis._documentId || classThis._documentId === '') {
						reject(this._translate.instant('REST_API.RENDER.INVALID_REPORT_DATA_MESSAGE'));
						return;
					}
					classThis._reportId = id;
					resolve(data.arrayBuffer());
				},
				(err: any) => {
					reject(err);
				});
		});
	}

	/**
	 * Exports report to return URL.
	 * @param extensionName - possible extension name: 'PDF', 'Word', 'Image', 'Mht', 'Excel'
	 * @param printDialog - open print dialog, works for PDF only
	 */
	export(extensionName: string, printDialog: boolean): Promise<string> {
		if (!this._reportId || !this._documentId)
			return Promise.reject<string>(this._translate.instant('REST_API.EXPORT.NOT_RENDERED_REPORT_MESSAGE'));

		var extensionSettings = this.getExportSettings(extensionName, true, printDialog);
		return this.handleRepeatRequest(this._reportId, 'reports', 'export', 'reports/jobs/export', '',
			{ ExtensionName: extensionName, ExtensionSettings: extensionSettings, DocumentId: this._documentId, UseCache: true }).then(
			(data: Response) => {
				return this.getFullCachePath(data.text());
			});
	}

	/**
	 * Gets all printers.
	 */
	get printers(): Promise<Printer[]> {
		var classThis = this;
		return new Promise<Printer[]>((resolve, reject) => {
			classThis.sendGetRequest('api/printers').then(
				(value: Response) => {
					var printers = <Array<any>>value.json();
					if (printers.length === 0) {
						reject(classThis._translate.instant('ERRORS.NO_PRINTERS'));
						return;
					}
					classThis.sendGetRequest('api/agents').then(
						(value2: Response) => {
							var agents = <Array<any>>value2.json();
							if (agents.length === 0) {
								reject(classThis._translate.instant('ERRORS.NO_PRINTERS'));
								return;
							}
							var printersList = new Array<Printer>();
							for (var printer of printers) {
								var name = printer['Name'];
								var id = printer['_id'];
								var options = printer['Options'];
								var host: string = null;
								for (var agentId of <Array<string>>printer['AgentIds']) {
									for (var agent of agents) {
										if (agent['Id'] !== agentId)
											continue;
										host = agent['Host'];
										var title = classThis._translate.instant('REST_API.PRINTER_NAME', { printer: name, host: host });
										printersList.push(new Printer(
											title,
											id,
											agentId,
											options['CanDuplex'],
											options['TwoSided'],
											options['MaximumCopies'],
											options['SupportsColor'],
											options['PaperSizes'],
											options['PaperSources']
										));
									}
								}
							}
							if (printersList.length === 0) {
								reject(classThis._translate.instant('ERRORS.NO_PRINTERS'));
								return;
							}
							resolve(printersList);
						},
						(err: any) => {
							reject(err);
						});
				},
				(err: any) => {
					reject(err);
				});
		});
	}

	/**
	 * Start printing task.
	 */
	print(agentId: string, printerId: string, twoSided: TwoSided, color: boolean, copies: number, paperName: string, paperSourceName: string): Promise<void> {
		if (!this._reportId)
			return Promise.reject<void>(this._translate.instant('REST_API.EXPORT.NOT_RENDERED_REPORT_MESSAGE'));

		return this.handleRepeatRequest(this._reportId, 'reports', 'print', 'reports/jobs/print', '',
			{ AgentId: agentId, PrintingOptions: { PrinterId: printerId, Twosided: TwoSided[twoSided], Color: color, Copies: copies, PaperName: paperName, PaperSourceName: paperSourceName } }).then((data: Response) => { });
	}

	/**
	 * Gets the link to the document that is cached on the server.
	 * @param path - relative path to the exported document on the server
	 */
	private getFullCachePath(path: string) {
		var server = localStorage.getItem('ars.server');
		return server + 'cache/' + path;
	}

	/**
	 * Sends a request and waiting for its execution.
	 * See more info here: http://arhelp.grapecity.com/webhelp/AR12Server/WebAPI/index.html
	 * @param id - report identifier (report ID, dataset ID, etc)
	 * @param apiType - type of API for first request ('accounts', 'agents', reports', 'caching', etc)
	 * @param requestType - type of operation for first request  (POST, e.g.: 'update', 'export', 'snapshot', etc)
	 * @param apiRepeatType -   type of API for second request (GET, e.g.: 'reports/jobs/print', reports/jobs/render', reports/jobs/export')
	 * @param requestRepeatType -  type of operation for second request
	 * @param settings - object containing data to be sent to the server (e.g.: report rendering settings)
	 * @param responseType - the type of data that you're expecting back from the server
	 */
	private handleRepeatRequest(id: string, apiType: string, requestType: string, apiRepeatType: string, requestRepeatType: string, settings: any, responseType?: ResponseContentType): Promise<Response> {
		var classThis = this;
		return new Promise<Response>((resolve, reject) => {
			classThis.sendPostRequest('api/' + apiType + '/' + id + '/jobs/' + requestType, settings, responseType).then(
				(value: Response) => {
					function downloadData(data: Response) {
						if (data.status === 200) {
							resolve(data);
							return;
						}
						// something wrong
						if (data.status !== 202) {
							reject(data.text());
							return;
						}
						// repeat request
						var requestId = JSON.parse(data.text('iso-8859'))['RequestId'];
						classThis.sendGetRequest('api/' + apiRepeatType + '/' + requestId + requestRepeatType, responseType).then(
							(repeatValue: Response) => {
								// recursivly calls this method till content is ready
								downloadData(repeatValue);
							},
							(err: any) => {
								reject(err);
							});
					}

					downloadData(value);
				},
				(err: any) => {
					reject(err);
				});
		});
	}

	/**
	 * Returns default settings for export.
	 * @param extensionName - name of file extension.  Valid values: 'PDF', 'Word', 'Image', 'Mht' or 'Excel'
	 * @param forExport - open for export
	 * @param printDialog - open for print
	 */
	private getExportSettings(extensionName: string, forExport: boolean, printDialog: boolean): any {
		var extensionSettings =
			{
				StreamingEnabled: false
			};
		switch (extensionName) {
			case 'PDF':
				extensionSettings['Application'] = this._translate.instant('REST_API.EXPORT.AUTHOR');
				extensionSettings['Author'] = this._translate.instant('REST_API.EXPORT.APPLICATION');
				extensionSettings['OptimizeStatic'] = false;
				if (printDialog) extensionSettings['PrintOnOpen'] = true;
				break;
			case 'Word':
				extensionSettings['Author'] = this._translate.instant('REST_API.EXPORT.AUTHOR');
				extensionSettings['FileFormat'] = "OOXML";
				break;
			case 'Image':
				extensionSettings['Compression'] = "Lzw";
				extensionSettings['ImageType'] = "TIFF";
				break;
			case 'Mht':
				extensionSettings['MhtOutput'] = true; // it is important, otherwise this export returns a lot of files
				break;
			case 'Excel':
				extensionSettings['MultiSheet'] = false;
				break;
		}
		if (printDialog)
			extensionSettings['Target'] = 'Paper';
		else if (forExport)
			extensionSettings['Target'] = 'Export';
		else
			extensionSettings['Target'] = 'Screen';
		return extensionSettings;
	}

	/**
	 * Parse an error.
	 * @param error - object that contains an error
	 */
	private parseError(error: any){
		if (error instanceof Response)
			return this.parseResponseException(error);

		if (error instanceof DOMException)
			return this.parseDOMException(error);

		// IE-specific
		// See details here: https://forums.asp.net/post/4983054.aspx
		if (error.hasOwnProperty('message') && error.message === "Access is denied.\r\n")
			return this._translate.instant('REST_API.INVALID_URL_MESSAGE');

		return this._translate.instant('ERRORS.UNKNOWN_ERROR');
	}

	/**
	 * Parse an error.
	 * @param error - response that contains an error
	 */
	private parseResponseException(error: Response): any {
		if (error.url === null || error.status == 404)
			return this._translate.instant('REST_API.INVALID_URL_MESSAGE');

		// web error
		var message = error.statusText;
		var errorAsText = error.text('iso-8859');

		if (errorAsText.indexOf('{') === 0 && errorAsText.lastIndexOf('}') === errorAsText.length - 1) {
			var data = JSON.parse(errorAsText);
			// RESTful API error
			if (data['ErrorCode']) {
				var errorCodeKey = 'ERRORS.ARS.' + data['ErrorCode'];
				message = this._translate.instant(errorCodeKey);
				if (message === errorCodeKey && data['Error']) {
					message = data['Error'];
				}
			} else if (data['Error']) {
				message = data['Error'];
			} else {
				message = this._translate.instant('ERRORS.UNKNOWN_ERROR');
			}
		}

		return message || this._translate.instant('ERRORS.UNKNOWN_ERROR');
	}

	/**
	 * Parse DOMException
	 * @param exception - exception that contains an error
	 */
	private parseDOMException(exception: DOMException){
		if (exception.name === 'NetworkError')
		{
			return this._translate.instant('REST_API.INVALID_URL_MESSAGE');
		}

		return exception.message;
	}

	/**
	 * Sends POST request
	 * @param command - request path
	 * @param data - object containing data to be sent to the server
	 * @param responseType - the type of data that you're expecting back from the server
	 */
	private sendPostRequest(command: string, data: any, responseType?: ResponseContentType): Promise<any> {
		var token = localStorage.getItem('ars.token');
		if (!token)
			return Promise.reject(this._translate.instant('REST_API.CREDENTIALS.NOT_LOGGED_MESSAGE'));
		var server = localStorage.getItem('ars.server');
		if (!server)
			return Promise.reject(this._translate.instant('REST_API.CREDENTIALS.NOT_LOGGED_MESSAGE'));

		// send HTTP post request
		this._headers.set('AuthToken', token.toString());
		return this._http.post(server + command, data, new RequestOptions({ headers: this._headers, responseType: responseType }))
			.toPromise()
			.catch((reason: any) => Promise.reject(this.parseError(reason)));
	}

	/**
	 * Sends GET request
	 * @param command - request path
	 * @param responseType - the type of data that you're expecting back from the server
	 */
	private sendGetRequest(command: string, responseType?: ResponseContentType): Promise<any> {
		var token = localStorage.getItem('ars.token');
		if (!token)
			return Promise.reject(this._translate.instant('REST_API.CREDENTIALS.NOT_LOGGED_MESSAGE'));
		var server = localStorage.getItem('ars.server');
		if (!server)
			return Promise.reject(this._translate.instant('REST_API.CREDENTIALS.NOT_LOGGED_MESSAGE'));

		// send HTTP get request
		this._headers.set('AuthToken', token.toString());
		return this._http.get(server + command, new RequestOptions({ headers: this._headers, responseType: responseType }))
			.toPromise()
			.catch((reason: any) => Promise.reject(this.parseError(reason)));
	}
}
