require('pdfjs-dist/webpack');

import { Component, OnInit, AfterContentInit, ViewContainerRef } from '@angular/core';
import { DomSanitizer, SafeResourceUrl } from '@angular/platform-browser';
import { TranslateService } from '@ngx-translate/core';
import { MdDialog, MdDialogConfig, MdDialogRef } from '@angular/material';

import { PrintServerComponent } from '../print-server/print-server.component'
import { AuthService } from '../auth.service'
import { ArsRestService, Report } from '../ars-rest.service'
import { PdfPrintService } from '../pdf-print.service'

/**
 * Main preview.
 */
@Component({
	selector: 'app-preview',
	templateUrl: './preview.component.html',
	styleUrls: ['./preview.component.scss']
})
export class PreviewComponent implements OnInit, AfterContentInit {

	// initial state check
	private _isLogged: boolean;

	// reports list
	public isReportsLoaded: boolean;
	public reportsList: Array<Report>;

	// selected report values
	public currentReport: Report;
	public isCurrentReportLoaded: boolean;
	public currentReportData: ArrayBuffer;
	public currentReportDataUrl: SafeResourceUrl;

	// run report options
	public chosenOperation: string;
	public selectedExportFormat: string;
	public operationInProgress: boolean;

	private _dialogRef: MdDialogRef<PrintServerComponent>;

	constructor(
		public dialog: MdDialog, // required for dialog: http://stackoverflow.com/a/40185852
		public viewContainerRef: ViewContainerRef, // required for dialog: http://stackoverflow.com/a/40185852

		private _authService: AuthService,
		private _restService: ArsRestService,
		private _printService: PdfPrintService,
		private _domSanitizer: DomSanitizer,
		private _translate: TranslateService) {

		this.reportsList = new Array<Report>();
		this.chosenOperation = 'save_as';
		this.selectedExportFormat = 'PDF';
		this.currentReportDataUrl = this._domSanitizer.bypassSecurityTrustResourceUrl(encodeURI(''));
	}

	/**
	 * Checks credentials and required authentication if necessary.
	 */
	ngOnInit() {
		this._isLogged = this._authService.checkCredentials();
	}

	/**
	 * Loads reports without parameters from server.
	 */
	ngAfterContentInit() {
		if (!this._isLogged)
			return;

		// checks credentials to ensure if they are valid
		// it is important thing here because we cannot avoid this method
		this._restService.isCredentialsValid().then(
			() => {
				// loads reports list
				this._restService.reports.then(
					(data: Report[]) => {
						for (var item of data) {
							if (item.isParametrized)
								continue; // skip all reports with parameters
							this.reportsList.push(item);
						}
						this.isReportsLoaded = true;
					},
					(err: any) => {
						this.showErrorMessage(err);
					});
			},
			(err: any) => {
				// just force logout
				this._authService.logout();
			});
	}

	/**
	 * Logout from ActiveReports Server.
	 */
	logout(): void {
		this._authService.logout();
	}

	/**
	 * Loads selected report as PDF.
	 */
	loadReport(report: Report): void {
		if (this.currentReport && this.currentReport.id === report.id)
			return; // already loaded

		this.currentReport = report;
		this.isCurrentReportLoaded = false;
		this._restService.render(this.currentReport.id, 'PDF', false, null).then(
			(data: ArrayBuffer) => {
				if (!!data && data.byteLength > 0) {
					this.currentReportData = data;

					var file = new Blob([data], { type: 'application/pdf' });
					var fileURL = URL.createObjectURL(file);
					this.currentReportDataUrl = this._domSanitizer.bypassSecurityTrustResourceUrl(encodeURI(fileURL));
				}
				this.isCurrentReportLoaded = true;
			},
			(err: any) => {
				this.currentReport = null;
				this.showErrorMessage(err);
			}
		);
	}

	/**
	 * Exports/prints selected report.
	 */
	runReport(): void {
		switch (this.chosenOperation) {
			case 'save_as':
				this.saveAs(this.selectedExportFormat);
				break;
			case 'print_client':
				this.printOnClient();
				break;
			case 'print_server':
				this.openPrintDialog();
				break;
			default:
				throw new Error('Unknown operation: ' + this.chosenOperation);
		}
	}

	private showErrorMessage(err: string): void {
		alert(err);
	}

	/**
	 * Export selected report.
	 * @param format - possible extension name: 'PDF', 'Word', 'Image', 'Mht', 'Excel'
	 */
	private saveAs(format: string): void {
		this.operationInProgress = true;
		this._restService.export(format, false).then(
			(data: string) => {
				this.operationInProgress = false;
				window.location.href = data + '&Attachment=1';
			},
			(err: any) => {
				this.operationInProgress = false;
				this.showErrorMessage(err);
			});
	}

	/**
	 * Print selected report.
	 */
	private printOnClient(): void {
		this.operationInProgress = true;

		// Export the report and then print the exported data
		this._restService.export('PDF', true).then(
			(data: string) => {
				this._printService.print(data);
			},
			(err: any) => {
				this.operationInProgress = false;
				this.showErrorMessage(err);
			}).then(() => {
				this.operationInProgress = false;
			},
			(err: any) => {
				this.operationInProgress = false;
				this.showErrorMessage(err);
			});
	}

	/**
	 * Shows the ARS printing options dialog.
	 */
	private openPrintDialog(): void {
		let config = new MdDialogConfig();
		config.viewContainerRef = this.viewContainerRef;
		this._dialogRef = this.dialog.open(PrintServerComponent, config);
		this._dialogRef.afterClosed().subscribe(result => this._dialogRef = null);
	}
}
