import { Component, AfterContentInit } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { MdDialogRef } from '@angular/material';

import { ArsRestService, Printer, TwoSided } from '../ars-rest.service'

@Component({
	selector: 'app-print-server',
	templateUrl: './print-server.component.html',
	styleUrls: ['./print-server.component.scss']
})
export class PrintServerComponent implements AfterContentInit {

	public printer: number = -1; // current value
	public printers: Array<Printer> = new Array<Printer>(); // value from printer
	public isPrintersLoading: boolean = true;

	public color: boolean = false; // current value
	public supportsColor: boolean = false; // value from printer

	public copies: number = 1; // current value
	public maximumCopies: number = 1; // value from printer
	public copiesNumbers: Array<number> = new Array<number>();

	public paperSourceName: string = ''; // current value
	public paperSources: Array<string> = new Array<string>(); // value from printer

	public paperName: string = ''; // current value
	public paperSizes: Array<string> = new Array<string>(); // value from printer

	public twoSided: TwoSided = TwoSided.OneSided; // current value
	public canDuplex: boolean = false; // value from printer

	public printing: boolean;

	constructor(
		public dialogRef: MdDialogRef<PrintServerComponent>,
		private _restService: ArsRestService,
		private _translate: TranslateService) {
	}

	/**
	 * Loads printers-related information.
	 */
	ngAfterContentInit() {
		// loads reports list
		this._restService.printers.then(
			(data: Printer[]) => {
				this.isPrintersLoading = false;
				this.printers = data;
				if (this.printers.length > 0) {
					this.selectPrinter(0); // select first printer
				}
				else {
					this.showErrorMessage(this._translate.instant('ERRORS.NO_PRINTERS'));
					this.dismiss(); // close if no printers are found
				}
			},
			(err: any) => {
				this.isPrintersLoading = false;
				this.showErrorMessage(err);
				this.dismiss();
			});
	}

	/**
	 * Updates UI.
	 */
	selectPrinter(printer: number) {
		if (this.printer === printer)
			return;

		this.printer = printer;
		var currentPrinter = this.printers[printer];

		this.supportsColor = currentPrinter.supportsColor;
		if (!this.supportsColor)
			this.color = false;

		this.maximumCopies = currentPrinter.maximumCopies;
		this.copiesNumbers = new Array<number>(this.maximumCopies).fill(0).map((x, i) => i + 1);
		if (this.copies > this.maximumCopies)
			this.copies = this.maximumCopies;
		if (this.copies < 1)
			this.copies = 1;

		this.paperSources = currentPrinter.paperSources;
		if (!this.paperSources.includes(this.paperSourceName))
			this.paperSourceName = this.paperSources.length > 0 ? this.paperSources[0] : ''; // just reset

		this.paperSizes = currentPrinter.paperSizes;
		if (!this.paperSizes.includes(this.paperName))
			this.paperName = this.paperSizes.length > 0 ? this.paperSizes[0] : ''; // just reset

		this.canDuplex = currentPrinter.canDuplex;

		this.twoSided = currentPrinter.twoSided;
		if (!this.canDuplex)
			this.twoSided = TwoSided.OneSided;
	}

	/**
	 * Dismiss this dialog.
	 */
	dismiss(): void {
		this.dialogRef.close();
	}

	/**
	 * Print on server.
	 */
	print(): void {
		var currentPrinter = this.printers[this.printer];
		this.printing = true;
		this._restService.print(currentPrinter.agentId, currentPrinter.id, this.twoSided, this.color, this.copies, this.paperName, this.paperSourceName).then(
			() => {
				this.printing = false;
				this.dialogRef.close();
			},
			(err: any) => {
				this.printing = false;
				this.showErrorMessage(err);
				this.dismiss();
			});
	}

	private showErrorMessage(err: string): void {
		alert(err);
	}
}
